﻿var viewModel;

$(function () {
    viewModel = {
        comments: ko.observableArray(),
        remove: remove,
        exists: exists,
        errorMessage: ko.observable(""),
        updateCommentsList: updateCommentsList,
        validate: validateComments,
        save: saveComments
    };

    var serviceName = 'breeze/Breeze';
    var vm = viewModel;
    var manager = new breeze.EntityManager(serviceName);
    ko.applyBindings(vm, document.getElementById('comments'));
    getComments();

    //Breeze calls to WebAPI
    function getComments() {
        var query = breeze.EntityQuery.from("Comments").withParameters({ UserId: document.getElementById('hiddenId').value });
        return manager.executeQuery(query)
            .then(getSucceeded).fail(failed);
    }

    function getSucceeded(data) {
        var count = data.results.length;
        log("Retrieved Comments: " + count);
        if (!count) {
            log("No Comments");
            return;
        }
        vm.comments(data.results);
    }

    function saveComments() {
        manager.saveChanges()
            .then(saveSucceeded)
            .fail(failed);
    }

    function saveSucceeded() {
        log("No Comments");
    }

    function failed(error) {
        vm.errorMessage(error);
    }

    //Track changes in the UI
    function updateCommentsList(selectedValue, selectedText) {
        if (exists(selectedValue)) {
            var comment = remove(selectedValue);
            comment.entityAspect.setDeleted();
        } else {
            //if we are adding a comment that we deleted for one of the items,
            //pull the deleted comment back in.
            //otherwise, just create a new one
            var deleted = manager.getChanges().filter(function (e) {
                return e.FunStuffId() == selectedValue;  //double equals (not triple) since one is int and other is string
            })[0];  //note .filter won't work in IE8 or earlier
            var newSelection;
            if (deleted) {
                newSelection = deleted;
                deleted.entityAspect.rejectChanges();
            } else {
                newSelection = manager.createEntity('FunStuffComment', {
                    'UserId': document.getElementById('hiddenId').value,
                    'FunStuffId': selectedValue,
                    'FunStuffName': selectedText,
                    'Comment': ""
                });
            }
            viewModel.comments.push(newSelection);
        }
    }

    function remove(stuffId) {
        var comments = vm.comments();
        var selected = comments.filter(function (item) {
            return stuffId == item.FunStuffId();
        })[0];
        if (selected) {
            comments.splice(comments.indexOf(selected), 1); //find it then remove the first 1
            vm.comments.valueHasMutated();  //does this now work without this line?
        }
        return selected;
    };

    function exists(stuffId) {
        var existingItem = ko.utils.arrayFirst(vm.comments(), function (item) {
            return stuffId == item.FunStuffId();
        });
        return existingItem != null;

    };


});

function validateComments() {
    //always true
    return true;

    //for testing:randomly return true or false to validate that false works in UI
    var d = new Date();
    var s = d.getSeconds();
    if (s <= 15 || (s >= 30 && s <= 40) || s > 50)
        return true;
    else { return false; }
}


function log(text) {
    console.log(text);
    document.getElementById('log').innerHTML += '<li>' + text + '</li>';
}
